@@ -54,10 +54,6 @@
  */
 #define PROCESS_OUTSTANDING_TIMEOUT 7200
 
-/* The maximum allowed size of a report file, if not overridden
- */
-#define MAX_REPORT_FILESIZE 1000000000
-
 /* If true, we have an active Internet connection. True by default in case we
  * can't bring up GNetworkMonitor */
 static gboolean online_state = TRUE;
@@ -82,6 +78,9 @@ static long verifypeer = 1;
 /* The URL for sending the initial crash report */
 static char* crash_db_submit_url = NULL;
 
+/* OOPS ID of the last successfully-uploaded crash file */
+char* last_uploaded_oopsid = NULL;
+
 /* The file path and descriptor for our instance lock */
 static const char* lock_path = "/var/lock/whoopsie/lock";
 static int lock_fd = 0;
@@ -344,21 +343,14 @@ upload_report (const char* message_data, uint32_t message_len, struct response_s
         return response_code;
 }
 
-gsize
-get_report_max_size (void)
+void
+destroy_key_and_value (gpointer key, gpointer value, gpointer user_data)
 {
-    const char* value = NULL;
-    gsize max_size;
-
-    value = g_getenv ("REPORT_MAX_SIZE");
-    if (value == NULL)
-        return MAX_REPORT_FILESIZE;
-
-    max_size = atol(value);
-    if (max_size == 0)
-        return MAX_REPORT_FILESIZE;
-
-    return max_size;
+    if (key)
+        g_free (key);
+    /* The value may be "", which is allocated on the stack. */
+    if (value && *(char*)value != '\0')
+        g_free (value);
 }
 
 GHashTable*
@@ -379,48 +371,22 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
     gchar* token_p = NULL;
     char* key = NULL;
     char* value = NULL;
-    char* old_value = NULL;
     gchar* value_p = NULL;
     GError* err = NULL;
     gchar* end = NULL;
     size_t value_length;
     size_t value_pos;
-    struct stat st;
-    int fd;
-    int res;
-    gsize max_size;
 
     g_return_val_if_fail (report_path, NULL);
 
-    max_size = get_report_max_size();
-
-    fd = open(report_path, O_RDONLY | O_NOFOLLOW);
-    res = fstat(fd, &st);
-    if (res == -1) {
+    if (g_file_test (report_path, G_FILE_TEST_IS_SYMLINK) ||
+        !g_file_test (report_path, G_FILE_TEST_IS_REGULAR)) {
         g_set_error (error, g_quark_from_static_string ("whoopsie-quark"), 0,
-                     "%s could not be opened.", report_path);
-        if (fd >= 0)
-            close(fd);
+                     "%s is a symlink or is not a regular file.", report_path);
         return NULL;
     }
-
-    if(!S_ISREG(st.st_mode)) {
-        g_set_error (error, g_quark_from_static_string ("whoopsie-quark"), 0,
-                     "%s is not a regular file.", report_path);
-        close(fd);
-        return NULL;
-    }
-
-    /* Limit the size of a report */
-    if(st.st_size > max_size) {
-        g_set_error (error, g_quark_from_static_string ("whoopsie-quark"), 0,
-                     "%s is too big to be processed.", report_path);
-        close(fd);
-        return NULL;
-    }
-
     /* TODO handle the file being modified underneath us. */
-    fp = g_mapped_file_new_from_fd (fd, FALSE, &err);
+    fp = g_mapped_file_new (report_path, FALSE, &err);
     if (err) {
         g_set_error (error, g_quark_from_static_string ("whoopsie-quark"), 0,
                      "Unable to map report: %s", err->message);
@@ -430,17 +396,8 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
 
     contents = g_mapped_file_get_contents (fp);
     file_len = g_mapped_file_get_length (fp);
-
-    /* Check report size again to make sure */
-    if(file_len > max_size) {
-        g_set_error (error, g_quark_from_static_string ("whoopsie-quark"), 0,
-                     "%s is too big to be processed.", report_path);
-        goto error;
-    }
-
     end = contents + file_len;
-    hash_table = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                        g_free, g_free);
+    hash_table = g_hash_table_new (g_str_hash, g_str_equal);
     p = contents;
 
     while (p < end) {
@@ -475,17 +432,11 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
                                  0, "Report value too long.");
                     goto error;
                 }
-                g_hash_table_steal (hash_table, key);
                 value = g_realloc (value, value_pos + 1 + value_length + 1);
                 value_p = value + value_pos;
                 *value_p = '\n';
                 value_p++;
             } else {
-                /* Make sure we properly free the old empty string value */
-                old_value = g_hash_table_lookup (hash_table, key);
-                if (old_value)
-                    g_free (old_value);
-                g_hash_table_steal (hash_table, key);
                 value = g_realloc (value, value_length + 1);
                 value_p = value;
             }
@@ -496,13 +447,7 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
                 if (*c != '\t' && *c >= '\0' && *c < ' ')
                     *c = '?';
             value_p += value_length;
-            if (g_hash_table_contains (hash_table, key) == TRUE) {
-                g_set_error (error,
-                    g_quark_from_static_string ("whoopsie-quark"),
-                    0, "Report key must not be a duplicate.");
-                goto error;
-            }
-            g_hash_table_insert (hash_table, key, value ? value : g_strdup(""));
+            g_hash_table_insert (hash_table, key, value ? value : "");
             p = token_p + 1;
         } else {
             /* Reset the value pointer. */
@@ -572,17 +517,10 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
             }
             p = token_p + 1;
 
-            if (g_hash_table_contains (hash_table, key) == TRUE) {
-                g_set_error (error,
-                    g_quark_from_static_string ("whoopsie-quark"),
-                    0, "Report key must not be a duplicate.");
-                goto error;
-            }
-            g_hash_table_insert (hash_table, key, value ? value : g_strdup(""));
+            g_hash_table_insert (hash_table, key, value ? value : "");
         }
     }
     g_mapped_file_unref (fp);
-    close(fd);
 
     /* Remove entries that we don't want to send */
     if (!full_report) {
@@ -600,7 +538,8 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
                     continue;
             }
 
-            g_hash_table_iter_remove(&iter);
+            g_hash_table_iter_steal(&iter);
+            destroy_key_and_value(key, value, NULL);
         }
     }
 
@@ -608,10 +547,10 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
 
 error:
     if (hash_table) {
+        g_hash_table_foreach (hash_table, destroy_key_and_value, NULL);
         g_hash_table_destroy (hash_table);
     }
     g_mapped_file_unref (fp);
-    close(fd);
     return NULL;
 }
 
@@ -669,6 +608,20 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     return result_code == CURLE_OK && response_code == 200;
 }
 
+void unset_last_uploaded_oopsid ()
+{
+    if (last_uploaded_oopsid) {
+        g_free (last_uploaded_oopsid);
+        last_uploaded_oopsid = NULL;
+    }
+}
+
+void set_last_uploaded_oopsid (char* response)
+{
+    unset_last_uploaded_oopsid();
+    last_uploaded_oopsid = g_strdup_printf ("%.36s", response);
+}
+
 void
 handle_response (GHashTable* report, char* response_data)
 {
@@ -685,6 +638,7 @@ handle_response (GHashTable* report, char* response_data)
     if (command) {
         if (strcmp (command, "CORE") == 0) {
             log_msg ("Reported OOPS ID %.36s\n", response_data);
+            set_last_uploaded_oopsid(response_data);
             core = g_hash_table_lookup (report, "CoreDump");
             arch = g_hash_table_lookup (report, "Architecture");
             if (core && arch) {
@@ -695,10 +649,12 @@ handle_response (GHashTable* report, char* response_data)
                     log_msg ("Upload of the core dump failed.\n");
             } else if (strcmp (command, "OOPSID") == 0) {
                 log_msg ("Reported OOPS ID %.36s\n", response_data);
+                set_last_uploaded_oopsid(response_data);
             } else
                 log_msg ("Asked for a core dump that we don't have.\n");
         } else if (strcmp (command, "OOPSID") == 0) {
             log_msg ("Reported OOPS ID %.36s\n", response_data);
+            set_last_uploaded_oopsid(response_data);
         } else
             log_msg ("Got command: %s\n", command);
     }
@@ -762,6 +718,7 @@ parse_and_upload_report (const char* crash_file)
         destroy_response_string (&s);
     }
 
+    g_hash_table_foreach (report, destroy_key_and_value, NULL);
     g_hash_table_destroy (report);
 
     return success;
@@ -800,7 +757,7 @@ process_existing_files (const char* report_dir)
             g_free (crash_file);
             continue;
         } else if (online_state && parse_and_upload_report (crash_file)) {
-            if (!mark_handled (crash_file)) {
+            if (!mark_handled (crash_file, last_uploaded_oopsid)) {
                 log_msg ("Unable to mark report as seen (%s) removing it.\n", crash_file);
                 g_unlink (crash_file);
             }
